@@ -2,21 +2,20 @@
 
 =head1 NAME
 
-CUICollector.pl - Scrapes files from the MetaMapped Medline Baseline project to
-build a database of CUI bigram scores. 
+CUICollector.pl - Scrapes MetaMap Machine Output (MMO) files to build a database of CUI bigram scores. 
 
 =head1 SYNOPSIS
 
     $ perl CUICollector.pl --directory metamapped-baseline/2014/ 
-    CUICollector 0.04 - (C) 2015 Keith Herbert and Bridget McInnes, PhD
+    CUICollector 0.04 - (C) 2015 Keith Herbert and Bridget McInnes
     Released under the GNU GPL.
-    Connecting to CUI_DB on localhost
+    Connecting to database CUI_Bigrams on localhost
     Parsing file: /home/share/data/metamapped-baseline/2014/text.out_01.gz
     Parsing file: /home/share/data/metamapped-baseline/2014/text.out_02.gz
     Parsing file: /home/share/data/metamapped-baseline/2014/text.out_03.gz
     Parsing file: /home/share/data/metamapped-baseline/2014/text.out_02.gz
     Parsing file: /home/share/data/metamapped-baseline/2014/text.out_03.gz
-    Entering scores into CUI_DB
+    Entering scores into CUI_Bigrams
     ...
     Finished
 
@@ -51,18 +50,18 @@ Or just one:
 
 =head3 --database STRING        
 
-Database to contain the CUI bigram scores. DEFAULT: CUI_DB
+Database to contain the CUI bigram scores. DEFAULT: CUI_Bigrams
 
 If the database is not found in MySQL, CUICollector will create it for you. 
 
 =head3 --username STRING
 
-Username is required to access the bigram database on MySql. You will be prompted
+Username is required to access the CUI bigram database on MySql. You will be prompted
 for it if it is not supplied as an argument. 
 
 =head3 --password STRING
 
-Password is required to access the umls database on MySql. You will be prompted
+Password is required to access the CUI bigram database on MySql. You will be prompted
 for it if it is not supplied as an argument. 
 
 =head3 --hostname STRING
@@ -75,7 +74,7 @@ The port your mysql is using. DEFAULT: 3306
 
 =head3 --file_step INTEGER
 
-How many MetaMapped Medical Baseline files to read between writes to the database. 
+How many MetaMap files to read between writes to the database. 
 DEFAULT: 5
 
 MMO files can be rather large so setting a low file_step reduces the memory footprint of the script. However, setting a higher file_step reduces the number of write operations to the database.
@@ -105,19 +104,27 @@ The resulting database will have four tables:
 =over
 
 =item N_11
+
     cui_1   cui_2   n_11
+    
 This shows the count (n_11) for every time a particular CUI (cui_1) is immediately followed by another particular CUI (cui_2) in an utterance. 
 
 =item N_1P
+
     cui_1   n_1p
+    
 This shows the count (n_11) for every time a particular CUI (cui_1) is followed by any CUI in an utterance. 
 
 =item N_P1
+
     cui_2   n_p1
+    
 This shows the count (n_p1) for every time a particular CUI (cui_2) is immediately preceded by any CUI in an utterance. 
 
 =item N_PP
+
     n_pp
+    
 This single value is the total count of all cui_1, cui_2 bigram pairs. 
 
 =back
@@ -158,20 +165,24 @@ this program; if not, write to:
 use strict;
 use warnings;
 
-use Compress::Zlib; 
-use Data::Dumper;
-use DBI;
-use Getopt::Long;
+use Compress::Zlib;     # Read compress .gz files
+use Data::Dumper;       # Helps with debugging
+use DBI;                # Database stuff
+use Getopt::Long;       # Parse command line options
+use File::Type;         # Know when a file is compressed
+use Graph::Directed;    # Model an utternace as a directed graph
+use List::MoreUtils;    # these should be built-in
+
 
 use feature qw(say);
 
-$|=1;   # disables buffering to STDOUT
+$|=1;   # toggle buffering to STDOUT. Essential when CPU is fully worked.
 
 ###############################################################################
 # CONSTANT STRINGS
 ###############################################################################
 
-my $version = "0.19";
+my $version = "0.20";
 my $header = 
 "CUICollector $version - (C) 2015 Keith Herbert and Bridget McInnes, PhD\n"
 ."Released under the GNU GPL.";
@@ -187,15 +198,16 @@ my $usage = $header."\n"
 ."--hostname    Hostname for MySQL database. (Default=localhost)\n"
 ."--username    Username for MySQL access. (Optional, but will be prompted)\n"
 ."--password    Password for MySQL access. (Optional, but will be prompted)\n"
-."MetaMapped Baseline Files\n"
-."--file_step   How many files to read between database writes. (DEFAULT=5)\n"
+."MetaMap Machine Output File Options\n"
 ."--directory   Path to directory containing *only* MetaMap Baseline files\n"
 ."--files       Explicit list of one or more MetaMap Baseline files\n"
+."--file_step   How many files to read between database writes. (DEFAULT=5)\n"
+."--plain       All input files are uncompressed.\n"
 ."\nUSAGE EXAMPLES\n"
 ."Open directory ./metamap/ and write to database CUIs on localhost:\n"
 ."\tperl CUICollector.pl --database CUIs --directory metamap/\n\n"
-."Open single decompressed file test.mmb and write to default database:\n"
-."\tperl CUICollector.pl --plain --file test.mmb\n"
+."Open single file test.mmb and write to default database:\n"
+."\tperl CUICollector.pl --file test.mmb\n"
 ;
 
 ###############################################################################
@@ -213,13 +225,15 @@ my $username;
 my $password;
 
 my $file_step = 5;  # How many files to read between writes to database.
+my $gzipped = 1;    
 my @dirs    = ();   # Directories containing *only* MetaMap Baseline files
 my @files   = ();   # Explicit list of MetaMap Baseline files.
 
 GetOptions( 'debug'         => \$DEBUG, 
             'help'          => \$HELP,
             'verbose!'      => \$VERBOSE,
-            'quiet'         => sub { $VERBOSE = 0 }, 
+            'quiet'         => sub { $VERBOSE = 0 },
+            'plain'         => sub { $gzipped = 0 }, 
             'database=s'    => \$database,
             'hostname=s'    => \$hostname,
             'port=s'        => \$port,
@@ -262,11 +276,13 @@ for my $file (@files) {
     die "Cannot read $file" unless -r $file;
 }
 
+# Presorting makes the console output easier to read.
 @files = sort @files;
 
 ###############################################################################
 #                                   Main 
 ###############################################################################
+{
 say "Connecting to $database on $hostname" if $VERBOSE;
 my $dbh = open_mysql_database($database, $hostname, $port, $username, $password);
 
@@ -291,7 +307,7 @@ while (@files) {
 $dbh->disconnect;
 
 say "Finished." if $VERBOSE;
-
+}
 
 ###############################################################################
 #                           Database Subroutines
@@ -465,25 +481,76 @@ sub process_files {
         $$same_phrase{$cui_1}{$cui_2} = 1;
     };
     
-    
     for my $file (@files) {
-    
-        my $gz = gzopen("$file", "rb") or die "Cannot open $file\n";
-        say "Parsing file: $file" if $VERBOSE; 
         
-        # Count the bigrams in each utterance until the end of the file.         
-        until ($gz->gzeof()) {
-            count_bigrams( read_utterance($gz), $incrementor );
+        if( File::Type->mime_type($file) =~ /x-gzip/) {
+            process_gz_file($file, $incrementor);
+        }
+        else {
+            process_file($file, $incrementor);
         }
-        
-        $gz->gzclose();
     }
     
    return (\%bigrams, \%n1p, \%np1, $npp); 
 }
+###############################################################################
+sub process_file {
+    my $file = shift;           # text/plain file to process
+    my $incrementor = shift;    # ref to anon sub that updates bigram counts
+    
+    open(my $fh, '<', $file) or die "Cannot read $file: $!";    #fh = filehandle
+    say "Parsing uncompressed file: $file" if $VERBOSE;
+    
+    until (eof $fh ){
+        count_bigrams( read_utterance( $fh ), $incrementor );
+    }
+}
+
+sub process_gz_file {
+
+    my $file = shift;           # compressed .gz file to process
+    my $incrementor = shift;    # ref to anon sub that updates bigram counts
+        
+    my $gz = gzopen("$file", "rb") or die "Cannot read gzip compressed $file\n";
+    say "Parsing gzip compressed file: $file" if $VERBOSE; 
+    
+    # Count the bigrams in each utterance until the end of the file.         
+    until ($gz->gzeof()) {
+        count_bigrams( read_gz_utterance($gz), $incrementor );
+    }
+    
+    $gz->gzclose();
+}
 
 ###############################################################################
+# take filehandle as lexical variable
+# returns ref to array of phrases for single utterance
 sub read_utterance {
+    my $fh = shift;
+   
+    my @phrases_in_utterance;
+   
+   # The following loop will iterate over all the phrases for this utterance
+    while (<$fh>) {
+        
+        # Finish when we reach the End Of Utterance (EOU) marker
+        last if /^'EOU'/;
+            
+        # Skip all lines that aren't mappings for a phrase in the utterance
+        next unless /^mappings/;
+                
+        my @mappings = @{ get_mappings($_) };
+
+        push @phrases_in_utterance, \@mappings if @mappings;
+    }
+   
+    return \@phrases_in_utterance;
+}
+
+###############################################################################
+# takes the gzip "filehandle" object 
+# returns ref to array of phrases for single utterance
+sub read_gz_utterance {
     my $gz = shift;
    
     my @phrases;
@@ -497,20 +564,7 @@ sub read_utterance {
         # Skip all lines that aren't mappings for a phrase in the utterance
         next unless /^mappings/;
                 
-        # Break mappings into each possible mapping of phrase into CUIs
-        my @maps = split /map\(/, $_;
-
-        # Collect the CUIs in each possible mapping (assumes format 'C1234567')
-        # as a set of strings
-        my @mappings;           
-        for my $map (@maps) {
-            
-           my $CUI_string = join " ", ( $map =~ m/C\d{7}/g );
-           
-           say $CUI_string if $CUI_string and $DEBUG;
-           
-           push @mappings, $CUI_string if $CUI_string; 
-        }
+        my @mappings = @{ get_mappings($_) };
 
         push @phrases, \@mappings if @mappings;
     }
@@ -518,6 +572,28 @@ sub read_utterance {
     return \@phrases;
 }
 
+###############################################################################
+sub get_mappings {
+    my $mapping_string = shift;    # the line for which /^mappings/ is true
+    
+    # Break mappings into each possible mapping of phrase into CUIs
+    my @maps = split /map\(/, $mapping_string;
+
+    # Collect the CUIs in each possible mapping (assumes format 'C1234567')
+    # as a set of strings
+    my @mappings;           
+    for my $map (@maps) {
+        
+       my $CUI_string = join " ", ( $map =~ m/C\d{7}/g );
+       
+       say $CUI_string if $CUI_string and $DEBUG;
+       
+       push @mappings, $CUI_string if $CUI_string; 
+    }
+
+    return \@mappings;
+}
+
 ###############################################################################
 sub count_bigrams {
     my( $phrases_ref,   # reference to list of mappings for single utterance 
