@@ -12,7 +12,7 @@ score
 
 =head1 USAGE
 
-Usage: calculate-assocation.pl [OPTIONS] CUI1|TERM CUI2|TERM
+Usage: umls-assocation.pl [OPTIONS] CUI1|TERM CUI2|TERM
 
 =head1 INPUT
 
